from pathlib import Path

from flask_multipass import Multipass
from flask_sqlalchemy import SQLAlchemy
from flask_wtf.csrf import CSRFProtect
from webpack_boilerplate.config import setup_jinja2_ext

from . import cli
from .helpers import Flask, identity_handler, require_login


db = SQLAlchemy()
csrf = CSRFProtect()
multipass = Multipass()


def create_app():
    app = Flask(__name__, instance_relative_config=True)
    app.config.from_pyfile("config/config.py")

    app.cli.add_command(cli.alembic)

    db.init_app(app)
    # Ignore linter error unused export: we need to import the models to
    # register them with SQLAlchemy
    from . import models  # noqa: F401

    csrf.init_app(app)

    if app.config["REQUIRE_LOGIN"]:
        app.config["MULTIPASS_LOGIN_FORM_TEMPLATE"] = "login.html.j2"
        app.config["MULTIPASS_LOGIN_SELECTOR_TEMPLATE"] = "login_select.html.j2"
        app.config["MULTIPASS_SUCCESS_ENDPOINT"] = "views.index"
        multipass.identity_handler(identity_handler)
        multipass.init_app(app)

        for k, v in app.view_functions.items():
            if k.startswith("_flaskmultipass_saml_acs_"):
                csrf.exempt(v)

    app.config.update(
        {
            "WEBPACK_LOADER": {
                "MANIFEST_FILE": Path(app.static_folder).joinpath("manifest.json")
            }
        }
    )
    setup_jinja2_ext(app)

    instance_path = Path(app.instance_path)

    from . import views

    if app.config["REQUIRE_LOGIN"]:
        views.bp.before_request(require_login)
    app.register_blueprint(views.bp)

    from . import instance

    instance.bp.static_folder = instance_path / "data" / "static"
    instance.bp.template_folder = instance_path / "data" / "templates"
    if app.config["REQUIRE_LOGIN"]:
        instance.bp.before_request(require_login)
    app.register_blueprint(instance.bp)

    return app
