from . import db
from .models import Schild
from .helpers import Blueprint, get_template_attribute

from collections import namedtuple
from pathlib import Path

from flask import url_for

from flask_weasyprint import render_pdf

# bp.{static,template}_folder should actually be set here, but this is not
# possible without app context.
bp = Blueprint(
    "instance",
    __name__,
    url_prefix="/instance",
)


@bp.route("/schild/<uuid:ident>.html")
def schild_html(ident):
    schild = db.get_or_404(Schild, ident)
    return bp.render_template(schild.template, schild=schild)


@bp.route("/schild/<uuid:ident>.pdf")
def schild_pdf(ident):
    return render_pdf(url_for(".schild_html", ident=ident))


@bp.route("/sample/<template>.html")
def sample_html(template):
    return bp.render_template(template)


@bp.route("/sample/<template>.pdf")
def sample_pdf(template):
    return render_pdf(url_for(".sample_html", template=template))


Template = namedtuple("Template", "name description")


def list_templates():
    schild = Schild()
    loader = bp.jinja_loader
    for t in loader.list_templates():
        if t.startswith("_"):
            continue
        yield Template(
            name=t,
            description=get_template_attribute(
                bp.real_template_name(t),
                "description",
                vars=dict(schild=schild),
                default=None,
            ),
        )


def list_images():
    base = Path(bp.static_folder) / "img"
    for f in base.iterdir():
        yield f.name
